@@ -55,8 +55,7 @@ static void entry_menu_item_class_init (EntryMenuItemClass *klass);
 static void entry_menu_item_init       (EntryMenuItem *self);
 static void entry_menu_item_dispose    (GObject *object);
 static void entry_menu_item_finalize   (GObject *object);
-static void handle_event               (DbusmenuMenuitem *mi, const gchar *name,
-										const GValue *value, guint timestamp);
+static void handle_event               (DbusmenuMenuitem * mi, const gchar * name, GVariant * value, guint timestamp);
 
 G_DEFINE_TYPE (EntryMenuItem, entry_menu_item, DBUSMENU_TYPE_MENUITEM);
 
@@ -122,14 +121,21 @@ entry_menu_item_new ()
    for the corresponding bottom-half service to act upon */
 static void
 handle_event (DbusmenuMenuitem *mi, const gchar *name,
-			  const GValue *value, guint timestamp)
+			  GVariant *value, guint timestamp)
 {
 	/* EntryMenuItemPrivate * priv = ENTRY_MENU_ITEM_GET_PRIVATE(self); */
-
-	g_debug ("handle_event");
 	if (g_strcmp0 (name, "send") == 0) {
+		gchar * msg = NULL;
+		if (g_variant_is_of_type (value, G_VARIANT_TYPE_VARIANT)) {
+			msg = g_strdup(g_variant_get_string(g_variant_get_variant (value), NULL));
+
+		} else if (g_variant_is_of_type (value, G_VARIANT_TYPE_STRING)) {
+			msg = g_strdup(g_variant_get_string(value, NULL));
+
+		}
+
 		MeGwibberService *me_gwibber_service = me_gwibber_service_get ();
-		me_gwibber_service_send (me_gwibber_service, g_value_get_string (value));
+		me_gwibber_service_send (me_gwibber_service, g_strdup(msg));
 		dbusmenu_menuitem_property_set (mi, ENTRY_MENUITEM_PROP_TEXT, "");
 	}
 
