@@ -24,13 +24,12 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <libdbusmenu-glib/menuitem.h>
 #include <libdbusmenu-gtk/menu.h>
 
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-bindings.h>
-
 #include <libindicator/indicator.h>
 #include <libindicator/indicator-object.h>
 #include <libindicator/indicator-service-manager.h>
@@ -40,7 +39,7 @@
 #include "about-me-menu-item.h"
 
 #include "dbus-shared-names.h"
-#include "me-service-client.h"
+#include "gen-me-service.xml.h"
 
 #define INDICATOR_ME_TYPE            (indicator_me_get_type ())
 #define INDICATOR_ME(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), INDICATOR_ME_TYPE, IndicatorMe))
@@ -72,27 +71,43 @@ INDICATOR_SET_TYPE(INDICATOR_ME_TYPE)
 /* Globals */
 static GtkImage * status_image = NULL;
 static GtkLabel *label = NULL;
-static DBusGProxy * status_proxy = NULL;
+static GDBusProxy * status_proxy = NULL;
+
+static GCancellable * status_proxy_cancel = NULL;
 
 static IdoEntryMenuItem *ido_entry = NULL;
 
+static const gchar *status_desc = NULL;
+
 /* Prototypes */
 static GtkLabel * get_label (IndicatorObject * io);
 static GtkImage * get_icon (IndicatorObject * io);
 static GtkMenu * get_menu (IndicatorObject * io);
+static const gchar * get_accessible_desc (IndicatorObject * io);
 
 static void indicator_me_class_init (IndicatorMeClass *klass);
 static void indicator_me_init       (IndicatorMe *self);
 static void indicator_me_dispose    (GObject *object);
 static void indicator_me_finalize   (GObject *object);
 static void connection_changed      (IndicatorServiceManager * sm, gboolean connected, gpointer userdata);
-static void status_icon_cb (DBusGProxy * proxy, char * icons, GError *error, gpointer userdata);
+static void receive_signal (GDBusProxy * proxy, gchar * sender_name, gchar * signal_name, GVariant * parameters, gpointer user_data);
+
 static gboolean new_entry_item (DbusmenuMenuitem * newitem,
                                 DbusmenuMenuitem * parent,
-                                DbusmenuClient   * client);
+                                DbusmenuClient   * client,
+				gpointer user_data);
+static gboolean new_about_me_item (DbusmenuMenuitem * newitem,
+                                DbusmenuMenuitem * parent,
+                                DbusmenuClient   * client,
+				gpointer user_data);
 static void     entry_activate_cb (GtkEntry *entry, DbusmenuMenuitem *mi);
-static void     entry_prop_change_cb (DbusmenuMenuitem *mi, gchar *prop, GValue *value, GtkEntry *entry);
+static void     entry_prop_change_cb (DbusmenuMenuitem *mi, gchar *prop, GVariant *value, GtkEntry *entry);
 static gboolean entry_hint_is_shown (GtkWidget *widget);
+static void status_proxy_cb (GObject * object, GAsyncResult * res, gpointer user_data);
+static void status_icon_changed (IndicatorMe * self, gchar * icon);
+static void status_icon_cb (GObject * object, GAsyncResult * res, gpointer user_data);
+static void username_changed (IndicatorMe * self, gchar * username);
+static void username_cb (GObject * object, GAsyncResult * res, gpointer user_data);
 
 G_DEFINE_TYPE (IndicatorMe, indicator_me, INDICATOR_OBJECT_TYPE);
 
@@ -108,6 +123,7 @@ indicator_me_class_init (IndicatorMeClass *klass)
 	io_class->get_label = get_label;
 	io_class->get_image = get_icon;
 	io_class->get_menu = get_menu;
+	io_class->get_accessible_desc = get_accessible_desc;
 
 	return;
 }
@@ -129,21 +145,65 @@ indicator_me_init (IndicatorMe *self)
 static void
 indicator_me_dispose (GObject *object)
 {
+        IndicatorMe * self = INDICATOR_ME(object);
+
+	if (self->service != NULL) {
+		g_object_unref(G_OBJECT(self->service));
+		self->service = NULL;
+	}
+
+	if (status_proxy != NULL) {
+		g_object_unref(status_proxy);
+		status_proxy = NULL;
+	}
+
+	if (status_proxy_cancel != NULL) {
+		g_cancellable_cancel(status_proxy_cancel);
+		g_object_unref(status_proxy_cancel);
+		status_proxy_cancel = NULL;
+	}
 
 	G_OBJECT_CLASS (indicator_me_parent_class)->dispose (object);
+
 	return;
 }
 
 static void
 indicator_me_finalize (GObject *object)
 {
-
 	G_OBJECT_CLASS (indicator_me_parent_class)->finalize (object);
 	return;
 }
 
 static void
-username_cb (DBusGProxy * proxy, char * username, GError *error, gpointer userdata)
+username_cb (GObject * object, GAsyncResult * res, gpointer user_data)
+{
+	GError * error = NULL;
+	gchar * username = NULL;
+	GVariant * result;
+
+	IndicatorMe * self = INDICATOR_ME (user_data);
+	g_return_if_fail(self != NULL);
+
+	result = g_dbus_proxy_call_finish(status_proxy, res, &error);
+	g_return_if_fail(status_proxy != NULL);
+
+	if (error != NULL) {
+		g_warning("Could not get the username %s", error->message);
+		g_error_free(error);
+		return;
+	}
+
+	g_variant_get(result, "(&s)", &username);
+
+	if (username != NULL) {
+		username_changed (self, username);
+	}
+
+}
+
+static void
+username_changed (IndicatorMe * self, gchar * username)
 {
   if (label == NULL) {
     label = GTK_LABEL(gtk_label_new(NULL));
@@ -157,17 +217,15 @@ username_cb (DBusGProxy * proxy, char * username, GError *error, gpointer userda
   } else {
     gtk_widget_hide(GTK_WIDGET(label));
   }
-
 }
 
 static GtkLabel *
 get_label (IndicatorObject * io)
 {
-  if (label == NULL) {
+	if (label == NULL) {
 		/* Create the label if it doesn't exist already */
-    username_cb (NULL, NULL, NULL, NULL);
-  }
-
+		username_changed (NULL, NULL);
+	}
 	return label;
 }
 
@@ -176,35 +234,104 @@ get_icon (IndicatorObject * io)
 {
 	if (status_image == NULL) {
 		/* Will create the status icon if it doesn't exist already */
-		status_icon_cb(NULL, DEFAULT_ICON, NULL, NULL);
+		status_icon_changed(NULL, DEFAULT_ICON);
 	}
 	return status_image;
 }
 
-static void
-status_icon_cb (DBusGProxy * proxy, char * icons, GError *error, gpointer userdata)
+static const gchar *
+get_accessible_desc (IndicatorObject * io)
 {
-	g_return_if_fail(icons != NULL);
-	g_return_if_fail(icons[0] != '\0');
+	if (label == NULL) {
+		/* Create the label if it doesn't exist already */
+		username_changed (NULL, NULL);
+	}
+	return g_strdup_printf("%s (%s)", gtk_label_get_text(label), status_desc);
+}
+
+static void
+status_icon_cb (GObject * object, GAsyncResult * res, gpointer user_data)
+{
+	GError * error = NULL;
+	gchar * icon = NULL;
+	GVariant * result;
+
+	IndicatorMe * self = INDICATOR_ME (user_data);
+	g_return_if_fail(self != NULL);
+
+	result = g_dbus_proxy_call_finish(status_proxy, res, &error);
+	g_return_if_fail(status_proxy != NULL);
+
+	if (error != NULL) {
+		g_warning("Could not get the status icon %s", error->message);
+		g_error_free(error);
+		return;
+	}
+
+	g_variant_get(result, "(&s)", &icon);
+
+	if (icon != NULL) {
+		status_icon_changed (self, icon);
+	}
+
+}
+
+static void
+status_icon_changed (IndicatorMe * self, gchar * icon)
+{
+	g_return_if_fail(icon != NULL);
+	g_return_if_fail(icon[0] != '\0');
 
 	if (status_image == NULL) {
-    status_image = indicator_image_helper (DEFAULT_ICON "-panel");
+		status_image = indicator_image_helper (DEFAULT_ICON "-panel");
 		gtk_widget_show(GTK_WIDGET(status_image));
 	}
 
-  gchar *panel_icon = g_strconcat (icons, "-panel", NULL);
-  indicator_image_helper_update (status_image, panel_icon);
-  g_free (panel_icon);
+	if (g_strcmp0(icon, "user-away") == 0) {
+		/* Translators: user status "Away" */
+		status_desc = _("Away");
+	} else if (g_strcmp0(icon, "user-invisible") == 0) {
+		/* Translators: user status "Invisible" */
+		status_desc = _("Invisible");
+	} else if (g_strcmp0(icon, "user-busy") == 0) {
+		/* Translators: user status "Busy" */
+		status_desc = _("Busy");
+	} else if (g_strcmp0(icon, "user-available") == 0) {
+		/* Translators: user status "Available" */
+		status_desc = _("Available");
+	} else if (g_strcmp0(icon, "user-indeterminate") == 0) {
+		/* Translators: user status "Unknown" */
+		status_desc = _("Unknown");
+	} else if (g_strcmp0(icon, "user-offline") == 0) {
+		/* Translators: user status "Offline" */
+		status_desc = _("Offline");
+	}
+
+	gchar *panel_icon = g_strconcat (icon, "-panel", NULL);
+	indicator_image_helper_update (status_image, panel_icon);
+	g_free (panel_icon);
 
 	return;
 }
 
+/* Receives all signals from the service, routed to the appropriate functions */
 static void
-status_icon_changed (DBusGProxy * proxy, gchar * icon, gpointer userdata)
+receive_signal (GDBusProxy * proxy, gchar * sender_name, gchar * signal_name,
+                GVariant * parameters, gpointer user_data)
 {
-	g_debug("Changing status icon: '%s'", icon);
+        IndicatorMe * self = INDICATOR_ME(user_data);
 
-	return status_icon_cb(proxy, icon, NULL, NULL);
+        if (g_strcmp0(signal_name, "StatusIconsChanged") == 0) {
+                gchar * icon;
+                g_variant_get (parameters, "(&s)", &icon);
+                status_icon_changed (self, icon);
+        }
+        else if (g_strcmp0(signal_name, "UserChanged") == 0) {
+                gchar * username;
+                g_variant_get (parameters, "(&s)", &username);
+                username_changed(self, username);
+        }
+	return;
 }
 
 static void
@@ -221,65 +348,97 @@ item_destroyed_cb (GtkObject *item,
 }
 
 static void
-connection_changed (IndicatorServiceManager * sm, gboolean connected, gpointer userdata)
+connection_changed (IndicatorServiceManager * self, gboolean connected, gpointer userdata)
 {
 	if (connected) {
 		if (status_proxy == NULL) {
-			GError * error = NULL;
-
-			DBusGConnection * sbus = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-
-			status_proxy = dbus_g_proxy_new_for_name_owner(sbus,
-                                                     INDICATOR_ME_DBUS_NAME,
-                                                     INDICATOR_ME_SERVICE_DBUS_OBJECT,
-                                                     INDICATOR_ME_SERVICE_DBUS_INTERFACE,
-                                                     &error);
-
-			if (error != NULL) {
-				g_warning("Unable to get status proxy: %s", error->message);
-				g_error_free(error);
+			if (status_proxy_cancel == NULL) {
+				status_proxy_cancel = g_cancellable_new();
 			}
+			g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+						G_DBUS_PROXY_FLAGS_NONE,
+						NULL,
+						INDICATOR_ME_DBUS_NAME,
+						INDICATOR_ME_SERVICE_DBUS_OBJECT,
+						INDICATOR_ME_SERVICE_DBUS_INTERFACE,
+						status_proxy_cancel,
+						status_proxy_cb,
+						userdata);
 
-      if (status_proxy == NULL) return;
-
-			dbus_g_proxy_add_signal(status_proxy, "StatusIconsChanged", G_TYPE_STRING, G_TYPE_INVALID);
-			dbus_g_proxy_connect_signal(status_proxy, "StatusIconsChanged", G_CALLBACK(status_icon_changed), NULL, NULL);
-
-			dbus_g_proxy_add_signal(status_proxy, "UserChanged", G_TYPE_STRING, G_TYPE_INVALID);
-			dbus_g_proxy_connect_signal(status_proxy, "UserChanged", G_CALLBACK(username_cb), NULL, NULL);
 		}
-
-		org_ayatana_indicator_me_service_status_icons_async(status_proxy, status_icon_cb, NULL);
-
-    /* query the service for the username to display */
-		org_ayatana_indicator_me_service_pretty_user_name_async(status_proxy, username_cb, NULL);
 	} else {
-    DbusmenuMenuitem *mi = g_object_get_data (G_OBJECT (ido_entry),
+		DbusmenuMenuitem *mi = g_object_get_data (G_OBJECT (ido_entry),
                                               "dbusmenuitem");
 
-    g_signal_handlers_disconnect_by_func (ido_entry,
+		g_signal_handlers_disconnect_by_func (ido_entry,
                                           G_CALLBACK (entry_activate_cb),
                                           mi);
 
-    g_signal_handlers_disconnect_by_func (ido_entry,
+		g_signal_handlers_disconnect_by_func (ido_entry,
                                           G_CALLBACK (entry_prop_change_cb),
                                           mi);
 
 		/* If we're disconnecting, go back to offline */
-		status_icon_cb(NULL, DEFAULT_ICON, NULL, NULL);
+		status_icon_changed(NULL, DEFAULT_ICON);
 
-    g_object_unref (status_proxy);
-    status_proxy = NULL;
+		g_object_unref (status_proxy);
+		status_proxy = NULL;
 
-    if (ido_entry != NULL) {
-      gtk_widget_destroy (GTK_WIDGET (ido_entry));
-      ido_entry = NULL;
-    }
+		if (ido_entry != NULL) {
+			gtk_widget_destroy (GTK_WIDGET (ido_entry));
+			ido_entry = NULL;
+		}
 	}
 
 	return;
 }
 
+/* Callback from trying to create the proxy for the service, this
+   could include starting the service.  Sometimes it'll fail and
+   we'll try to start that dang service again! */
+static void
+status_proxy_cb (GObject * object, GAsyncResult * res, gpointer user_data)
+{
+	GError * error = NULL;
+
+	g_return_if_fail(object != NULL);
+
+	GDBusProxy * proxy = g_dbus_proxy_new_for_bus_finish(res, &error);
+
+	if (status_proxy_cancel != NULL) {
+		g_object_unref(status_proxy_cancel);
+		status_proxy_cancel = NULL;
+	}
+
+	if (error != NULL) {
+		g_error("Could not grab DBus proxy for %s: %s", INDICATOR_ME_DBUS_NAME, error->message);
+		g_error_free(error);
+		return;
+	}
+
+	IndicatorMe * self = INDICATOR_ME (user_data);
+
+	/* Okay, we're good to grab the proxy at this point, we're
+	sure that it's ours. */
+	status_proxy = proxy;
+
+	g_signal_connect(status_proxy, "g-signal", G_CALLBACK(receive_signal), self);
+
+	/* Query to get the username */
+	g_debug("Get the username");
+	g_dbus_proxy_call(status_proxy, "PrettyUserName", NULL,
+		G_DBUS_CALL_FLAGS_NONE, -1, NULL,
+		(GAsyncReadyCallback) username_cb, self);
+
+	/* Query to get the status icon */
+	g_debug("Get the status icon");
+	g_dbus_proxy_call(status_proxy, "StatusIcons", NULL,
+		G_DBUS_CALL_FLAGS_NONE, -1, NULL,
+		(GAsyncReadyCallback) status_icon_cb, self);
+
+	return;
+}
+
 static void
 entry_hint_set_shown (GtkWidget *widget, gboolean flag)
 {
@@ -298,6 +457,19 @@ entry_set_style (GtkEntry *entry, GtkStateType state)
     gtk_widget_queue_draw (GTK_WIDGET (entry));
 }
 
+/* unconditionnaly show the hint in the entry */
+static void
+entry_hint_show_hint (GtkEntry *entry)
+{
+  g_return_if_fail (GTK_IS_ENTRY (entry));
+
+  const gchar *hint = g_object_get_data (G_OBJECT (entry),
+                                         DBUSMENU_ENTRY_MENUITEM_PROP_HINT);
+  gtk_entry_set_text (entry, hint);
+  entry_set_style (entry, GTK_STATE_INSENSITIVE);
+  entry_hint_set_shown (GTK_WIDGET (entry), TRUE);
+}
+
 static void
 entry_maybe_show_hint (GtkEntry *entry)
 {
@@ -322,9 +494,7 @@ entry_maybe_show_hint (GtkEntry *entry)
   } else {
     g_debug ("%s, nothing in the entry or not focused, so setting the hint to: %s", __func__, hint);
 
-    gtk_entry_set_text (entry, hint);
-    entry_set_style (entry, GTK_STATE_INSENSITIVE);
-    entry_hint_set_shown (GTK_WIDGET (entry), TRUE);
+    entry_hint_show_hint (entry);
   }
 }
 
@@ -375,29 +545,37 @@ entry_focus_grab_cb (GtkWidget *widget, GdkEventFocus *event)
 static gboolean
 entry_focus_out_cb (GtkWidget *widget, GdkEventFocus *event)
 {
+  g_return_val_if_fail (GTK_IS_ENTRY (widget), FALSE);
+
   GtkEntry *entry = GTK_ENTRY (widget);
 
   g_debug ("%s", __func__);
 
-  entry_maybe_show_hint (entry);
+  if (! entry_hint_is_shown (GTK_WIDGET (entry))) {
+    if (gtk_entry_get_text_length (entry) == 0) {
+      /* show the hint unconditionnaly, as the focus change
+         may not have propagated already and entry_maybe_show_hint
+         could get confused */
+      entry_hint_show_hint (entry);
+    }
+  }
 
   return FALSE;
 }
 
 static void
-entry_prop_change_cb (DbusmenuMenuitem *mi, gchar *prop, GValue *value, GtkEntry *entry)
+entry_prop_change_cb (DbusmenuMenuitem *mi, gchar *prop, GVariant *value, GtkEntry *entry)
 {
   g_return_if_fail (GTK_IS_ENTRY (entry));
 
 	if (g_strcmp0 (prop, DBUSMENU_ENTRY_MENUITEM_PROP_TEXT) == 0) {
-		gtk_entry_set_text (entry, g_value_get_string (value));
+		gtk_entry_set_text (entry, g_variant_get_string (value, NULL));
     if (gtk_entry_get_text_length (entry) == 0)
       entry_maybe_show_hint (entry);
 	}
 	if (g_strcmp0 (prop, DBUSMENU_ENTRY_MENUITEM_PROP_HINT) == 0) {
-    entry_set_hint (entry, g_value_get_string (value));
-    if (entry_hint_is_shown (GTK_WIDGET (entry)))
-        entry_maybe_show_hint (entry); /* visual update */
+    entry_set_hint (entry, g_variant_get_string (value, NULL));
+        entry_hint_show_hint (entry); /* visual update */
 	}
 }
 
@@ -407,19 +585,18 @@ entry_activate_cb (GtkEntry *entry, DbusmenuMenuitem *mi)
   g_return_if_fail (GTK_IS_ENTRY (entry));
   g_return_if_fail (DBUSMENU_IS_MENUITEM (mi));
 
-	GValue value = { 0 };
-	g_value_init (&value, G_TYPE_STRING);
-	g_value_set_static_string (&value, gtk_entry_get_text (entry));
+	GVariant * value = g_variant_new_string (gtk_entry_get_text (entry));
+	
+	g_debug ("user typed: %s", g_variant_get_string (value, NULL));
 
-	g_debug ("user typed: %s", g_value_get_string (&value));
-
-	dbusmenu_menuitem_handle_event (mi, "send", &value, gtk_get_current_event_time());
+	dbusmenu_menuitem_handle_event (mi, "send", value, gtk_get_current_event_time());
 }
 
 static gboolean
 new_entry_item (DbusmenuMenuitem * newitem,
                 DbusmenuMenuitem * parent,
-                DbusmenuClient   * client)
+                DbusmenuClient   * client,
+                gpointer user_data)
 {
 	g_return_val_if_fail(DBUSMENU_IS_MENUITEM(newitem), FALSE);
 	g_return_val_if_fail(DBUSMENU_IS_GTKCLIENT(client), FALSE);
@@ -434,7 +611,7 @@ new_entry_item (DbusmenuMenuitem * newitem,
 	}
 
   g_object_set_data (G_OBJECT (ido),
-                     "dbusmenitem",
+                     "dbusmenuitem",
                      newitem);
 
 	gtk_entry_set_width_chars (entry, 23); /* set some nice aspect ratio for the menu */
@@ -486,13 +663,13 @@ new_entry_item (DbusmenuMenuitem * newitem,
 /* Whenever we have a property change on a DbusmenuMenuitem
    we need to be responsive to that. */
 static void
-about_me_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, AboutMeMenuItem *item)
+about_me_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * value, AboutMeMenuItem *item)
 {
   g_return_if_fail (ABOUT_IS_ME_MENU_ITEM (item));
 
 	if (!g_strcmp0(prop, DBUSMENU_ABOUT_ME_MENUITEM_PROP_ICON)) {
     /* reload the avatar icon */
-    about_me_menu_item_load_avatar (item, g_value_get_string(value));
+    about_me_menu_item_load_avatar (item, g_variant_get_string(value, NULL));
   } else if (!g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_VISIBLE)) {
     /* normal, ignore */
 	} else {
@@ -505,7 +682,8 @@ about_me_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GValue * value, Ab
 static gboolean
 new_about_me_item (DbusmenuMenuitem * newitem,
                    DbusmenuMenuitem * parent,
-                   DbusmenuClient * client)
+                   DbusmenuClient * client,
+                   gpointer user_data)
 {
 	g_return_val_if_fail(DBUSMENU_IS_MENUITEM(newitem), FALSE);
 	g_return_val_if_fail(DBUSMENU_IS_GTKCLIENT(client), FALSE);
