@@ -130,21 +130,13 @@ me_gwibber_service_finalize (GObject *object)
 	return;
 }
 
-static gboolean
-check_account_send_enabled (GHashTable * account_table) {
-        return g_value_get_boolean(g_hash_table_lookup(account_table, "send_enabled"));
-}
-
 static void
 query_account_manager (MeGwibberService *self)
 {
 	g_return_if_fail (IS_ME_GWIBBER_SERVICE (self));
 
-	gpointer accounts_table;
-	gpointer account_table;
-	GHashTableIter accounts_iter;
-	gpointer account;
-	gboolean send_enabled;
+	GList* accounts_list;
+	GwibberAccount* account;
 
 	MeGwibberServicePrivate * priv = ME_GWIBBER_SERVICE_GET_PRIVATE(self);
 
@@ -152,26 +144,25 @@ query_account_manager (MeGwibberService *self)
 		g_warning ("no accounts, can't query for accounts");
 		goto reset_accounts_string;
 	}
+	accounts_list = gwibber_accounts_list (priv->me_gwibber_accounts);
 	
-        accounts_table = gwibber_accounts_list (priv->me_gwibber_accounts);
-        if (accounts_table == NULL) {
+        if (accounts_list == NULL) {
                 g_warning ("failed to get accounts list");
 				goto reset_accounts_string;
         }
 
-	g_hash_table_iter_init (&accounts_iter, accounts_table);
-
 	priv->has_configured_accounts = FALSE;
 	GList *list = NULL;
 
-	while (g_hash_table_iter_next (&accounts_iter, &account, &account_table)) {
-		send_enabled = check_account_send_enabled (account_table);
-		if (send_enabled) {
+	GList* account_it;
+	for (account_it = accounts_list; account_it != NULL; account_it = account_it->next) {
+		account = g_object_ref ((GwibberAccount*) account_it->data);
+		if (g_strcmp0(gwibber_account_get_send_enabled (account), "1") == 0) {
 			priv->has_configured_accounts = TRUE;
-			GValue *value = g_hash_table_lookup (account_table, "service");
-			if (value != NULL)
-				list = g_list_append (list,
-									  g_strdup (g_value_get_string (value)));
+			const gchar * service_name = gwibber_account_get_service (account);
+			if (service_name != NULL) {
+				list = g_list_append (list, g_strdup (service_name));
+			}
 		}
 	}
 
@@ -206,8 +197,6 @@ query_account_manager (MeGwibberService *self)
 				   ME_GWIBBER_SERVICE_SIGNAL_STATUS_CHANGED_ID,
 				   0, priv->status, TRUE);
 
-	g_hash_table_destroy(accounts_table);
-
 	return;
 
 reset_accounts_string:
@@ -280,7 +269,6 @@ static void
 me_gwibber_accounts_exists_cb (GwibberAccounts * me_gwibber_accounts, gboolean exists, gpointer userdata)
 {
 	MeGwibberServicePrivate * priv = ME_GWIBBER_SERVICE_GET_PRIVATE(ME_GWIBBER_SERVICE (userdata));
-
 	if (exists) {
 		if (priv->me_gwibber_accounts == NULL) {
 			priv->me_gwibber_accounts = gwibber_accounts_new ();;
@@ -304,16 +292,12 @@ MeGwibberService * me_gwibber_service_new (void){
 }
 
 void
-me_gwibber_service_send (MeGwibberService *self, const gchar *msg)
+me_gwibber_service_send (MeGwibberService *self, gchar * msg)
 {
 	g_return_if_fail (IS_ME_GWIBBER_SERVICE (self));
 
 	MeGwibberServicePrivate * priv = ME_GWIBBER_SERVICE_GET_PRIVATE(self);
 
-	GValue value = {0};
-	g_value_init(&value, G_TYPE_STRING);
-	g_value_set_string(&value, msg);
-
 	if (priv->me_gwibber_service == NULL) {
 		setup_service_proxies (self);
 		if (priv->me_gwibber_service == NULL) {
@@ -322,9 +306,7 @@ me_gwibber_service_send (MeGwibberService *self, const gchar *msg)
 		}
 	}
 
-	gwibber_service_send_message (priv->me_gwibber_service, g_value_get_string (&value));
-
-	g_value_unset(&value);
+	gwibber_service_send_message (priv->me_gwibber_service, msg);
 
 	return;
 }
