@@ -66,7 +66,7 @@ struct _MeGwibberService {
 GType me_gwibber_service_get_type (void) G_GNUC_CONST;
 MeGwibberService * me_gwibber_service_new (void);
 MeGwibberService * me_gwibber_service_get (void);
-void me_gwibber_service_send (MeGwibberService *self, const gchar *msg);
+void me_gwibber_service_send (MeGwibberService *self, gchar *msg);
 gboolean me_gwibber_service_has_configured_accounts (MeGwibberService *self);
 const gchar * me_gwibber_service_get_accounts_string (MeGwibberService *self);
 
