@@ -49,16 +49,18 @@
 #include "status-provider.h"
 #include "status-provider-mc5.h"
 #include "status-provider-pidgin.h"
+#include "status-provider-emesene.h"
 #include "status-provider-telepathy.h"
 
 #include "me-service-gwibber.h"
 #include "entry-menu-item.h"
 
 typedef StatusProvider * (*newfunc) (void);
-#define STATUS_PROVIDER_CNT   3
+#define STATUS_PROVIDER_CNT   4
 static newfunc status_provider_newfuncs[STATUS_PROVIDER_CNT] = {
 	status_provider_mc5_new,
 	status_provider_pidgin_new,
+	status_provider_emesene_new,
 	status_provider_telepathy_new
 };
 static StatusProvider * status_providers[STATUS_PROVIDER_CNT] = { 0 };
@@ -106,7 +108,7 @@ status_update (void) {
 		StatusProviderStatus localstatus = status_provider_get_status(status_providers[i]);
     g_debug ("provider[%d]: %s", i, status_strings[localstatus]);
 
-    if (localstatus >= STATUS_PROVIDER_STATUS_OFFLINE)
+    if (localstatus > STATUS_PROVIDER_STATUS_OFFLINE)
       /* offline and disconnected is similar for some providers
          so it's not meaningful to determine the 'indeterminate' status
       */
@@ -191,11 +193,12 @@ me_gwibber_status_update (MeGwibberService *instance, MeGwibberServiceStatus sta
     return;
   }
 
-  if (! dbusmenu_menuitem_property_get_bool (broadcast_field, DBUSMENU_MENUITEM_PROP_VISIBLE))
-    dbusmenu_menuitem_property_set_bool (broadcast_field, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
-
   dbusmenu_menuitem_property_set (broadcast_field, DBUSMENU_ENTRY_MENUITEM_PROP_HINT, me_gwibber_service_get_accounts_string (instance));
 
+  if (! dbusmenu_menuitem_property_get_bool (broadcast_field, DBUSMENU_MENUITEM_PROP_VISIBLE)) {
+    dbusmenu_menuitem_property_set_bool (broadcast_field, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
+  }
+
   return;
 }
 
@@ -265,14 +268,16 @@ build_accounts_menuitems (gpointer data)
     at_least_one = TRUE;
   }
 
-  if (program_is_installed ("ubuntuone-preferences")) {
+  /* Removed ubuntuone-control-panel-gtk from MeMenu (LP: #711233)
+  if (program_is_installed ("ubuntuone-control-panel-gtk")) {
     u1_accounts_mi = dbusmenu_menuitem_new();
     dbusmenu_menuitem_property_set(u1_accounts_mi, DBUSMENU_MENUITEM_PROP_LABEL,
                                    _("Ubuntu One..."));
     g_signal_connect(G_OBJECT(u1_accounts_mi), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-                     G_CALLBACK(spawn_on_activate_cb), "ubuntuone-preferences");
+                     G_CALLBACK(spawn_on_activate_cb), "ubuntuone-control-panel-gtk");
     at_least_one = TRUE;
   }
+  */
 
   if (at_least_one) {
     DbusmenuMenuitem *separator = dbusmenu_menuitem_new();
